'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var slatePluginsCommon = require('@udecode/slate-plugins-common');
var React = require('react');
require('slate-react');
var slate = require('slate');
var slateHyperscript = require('slate-hyperscript');
var server = require('react-dom/server');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function create$1(createState) {
  let state;
  const listeners = new Set();

  const setState = (partial, replace) => {
    const nextState = typeof partial === 'function' ? partial(state) : partial;

    if (nextState !== state) {
      const previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach(listener => listener(state, previousState));
    }
  };

  const getState = () => state;

  const subscribeWithSelector = (listener, selector = getState, equalityFn = Object.is) => {
    let currentSlice = selector(state);

    function listenerToAdd() {
      const nextSlice = selector(state);

      if (!equalityFn(currentSlice, nextSlice)) {
        const previousSlice = currentSlice;
        listener(currentSlice = nextSlice, previousSlice);
      }
    }

    listeners.add(listenerToAdd); // Unsubscribe

    return () => listeners.delete(listenerToAdd);
  };

  const subscribe = (listener, selector, equalityFn) => {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }

    listeners.add(listener); // Unsubscribe

    return () => listeners.delete(listener);
  };

  const destroy = () => listeners.clear();

  const api = {
    setState,
    getState,
    subscribe,
    destroy
  };
  state = createState(setState, getState, api);
  return api;
}

const useIsoLayoutEffect = typeof window === 'undefined' ? React.useEffect : React.useLayoutEffect;
function create(createState) {
  const api = typeof createState === 'function' ? create$1(createState) : createState;

  const useStore = (selector = api.getState, equalityFn = Object.is) => {
    const [, forceUpdate] = React.useReducer(c => c + 1, 0);
    const state = api.getState();
    const stateRef = React.useRef(state);
    const selectorRef = React.useRef(selector);
    const equalityFnRef = React.useRef(equalityFn);
    const erroredRef = React.useRef(false);
    const currentSliceRef = React.useRef();

    if (currentSliceRef.current === undefined) {
      currentSliceRef.current = selector(state);
    }

    let newStateSlice;
    let hasNewStateSlice = false; // The selector or equalityFn need to be called during the render phase if
    // they change. We also want legitimate errors to be visible so we re-run
    // them if they errored in the subscriber.

    if (stateRef.current !== state || selectorRef.current !== selector || equalityFnRef.current !== equalityFn || erroredRef.current) {
      // Using local variables to avoid mutations in the render phase.
      newStateSlice = selector(state);
      hasNewStateSlice = !equalityFn(currentSliceRef.current, newStateSlice);
    } // Syncing changes in useEffect.


    useIsoLayoutEffect(() => {
      if (hasNewStateSlice) {
        currentSliceRef.current = newStateSlice;
      }

      stateRef.current = state;
      selectorRef.current = selector;
      equalityFnRef.current = equalityFn;
      erroredRef.current = false;
    });
    const stateBeforeSubscriptionRef = React.useRef(state);
    useIsoLayoutEffect(() => {
      const listener = () => {
        try {
          const nextState = api.getState();
          const nextStateSlice = selectorRef.current(nextState);

          if (!equalityFnRef.current(currentSliceRef.current, nextStateSlice)) {
            stateRef.current = nextState;
            currentSliceRef.current = nextStateSlice;
            forceUpdate();
          }
        } catch (error) {
          erroredRef.current = true;
          forceUpdate();
        }
      };

      const unsubscribe = api.subscribe(listener);

      if (api.getState() !== stateBeforeSubscriptionRef.current) {
        listener(); // state has changed before subscription
      }

      return unsubscribe;
    }, []);
    return hasNewStateSlice ? newStateSlice : currentSliceRef.current;
  };

  Object.assign(useStore, api); // For backward compatibility (No TS types for this)

  useStore[Symbol.iterator] = function* () {
    console.warn('[useStore, api] = create() is deprecated and will be removed in v4');
    yield useStore;
    yield api;
  };

  return useStore;
}

function unwrapExports(x) {
  return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
  return module = {
    exports: {}
  }, fn(module, module.exports), module.exports;
}

var vanilla = createCommonjsModule(function (module, exports) {
  Object.defineProperty(exports, '__esModule', {
    value: true
  });

  function create(createState) {
    var state;
    var listeners = new Set();

    var setState = function setState(partial, replace) {
      var nextState = typeof partial === 'function' ? partial(state) : partial;

      if (nextState !== state) {
        var previousState = state;
        state = replace ? nextState : Object.assign({}, state, nextState);
        listeners.forEach(function (listener) {
          return listener(state, previousState);
        });
      }
    };

    var getState = function getState() {
      return state;
    };

    var subscribeWithSelector = function subscribeWithSelector(listener, selector, equalityFn) {
      if (selector === void 0) {
        selector = getState;
      }

      if (equalityFn === void 0) {
        equalityFn = Object.is;
      }

      var currentSlice = selector(state);

      function listenerToAdd() {
        var nextSlice = selector(state);

        if (!equalityFn(currentSlice, nextSlice)) {
          var previousSlice = currentSlice;
          listener(currentSlice = nextSlice, previousSlice);
        }
      }

      listeners.add(listenerToAdd); // Unsubscribe

      return function () {
        return listeners.delete(listenerToAdd);
      };
    };

    var subscribe = function subscribe(listener, selector, equalityFn) {
      if (selector || equalityFn) {
        return subscribeWithSelector(listener, selector, equalityFn);
      }

      listeners.add(listener); // Unsubscribe

      return function () {
        return listeners.delete(listener);
      };
    };

    var destroy = function destroy() {
      return listeners.clear();
    };

    var api = {
      setState: setState,
      getState: getState,
      subscribe: subscribe,
      destroy: destroy
    };
    state = createState(setState, getState, api);
    return api;
  }

  exports.default = create;
});
var createVanillaStore = unwrapExports(vanilla);
/**
 * Slate plugins vanilla store.
 * @see zustand vanilla store
 */

const slatePluginsStore = createVanillaStore(() => ({}));
/**
 * Slate plugins store.
 * @see zustand store
 */

create(slatePluginsStore);

const getSlatePluginWithOverrides = withOverrides => options => ({
  withOverrides: withOverrides(options)
});

slatePluginsStore.setState;

const s = slatePluginsStore;
s.getState;
s.setState;
const isElement = slate.Element.isElement;

slate.Text.isText;
/**
 * Get inline types from the plugins
 */


const getInlineTypes = (editor, plugins) => {
  return plugins.flatMap(p => {
    var _a, _b;

    return (_b = (_a = p.inlineTypes) === null || _a === void 0 ? void 0 : _a.call(p, editor)) !== null && _b !== void 0 ? _b : [];
  });
};

/**
 * Convert HTML string into HTML element.
 */
const htmlStringToDOMNode = rawHtml => {
  const node = document.createElement('body');
  node.innerHTML = rawHtml.replace(/(\r\n|\n|\r|\t)/gm, '');
  return node;
};

/**
 * Deserialize HTML to break line.
 */
const deserializeHTMLToBreak = node => {
  if (node.nodeName === 'BR') {
    return '\n';
  }
};

/**
 * Deserialize HTML to Element.
 */

const deserializeHTMLToElement = (editor, {
  plugins,
  element,
  children
}) => {
  let slateElement;
  let withoutChildren;
  plugins.some(({
    deserialize: pluginDeserializers
  }) => {
    const elementDeserializers = pluginDeserializers === null || pluginDeserializers === void 0 ? void 0 : pluginDeserializers(editor).element;
    if (!elementDeserializers) return;
    return elementDeserializers.some(deserializer => {
      const deserialized = deserializer.deserialize(element);
      if (!deserialized) return;
      slateElement = deserialized;
      withoutChildren = deserializer.withoutChildren;
      return true;
    });
  });

  if (slateElement) {
    let descendants = children;

    if (!descendants.length || withoutChildren) {
      descendants = [{
        text: ''
      }];
    }

    return slateHyperscript.jsx('element', slateElement, descendants);
  }
};

/**
 * Deserialize HTML body element to Fragment.
 */

const deserializeHTMLToFragment = ({
  element,
  children
}) => {
  if (element.nodeName === 'BODY') {
    return slateHyperscript.jsx('fragment', {}, children);
  }
};

/**
 * Deserialize HTML to TDescendant[] with marks on Text.
 * Build the leaf from the leaf deserializers of each plugin.
 */

const deserializeHTMLToMarks = (editor, {
  plugins,
  element,
  children
}) => {
  let leaf = {};
  plugins.forEach(({
    deserialize: pluginDeserializers
  }) => {
    const leafDeserializers = pluginDeserializers === null || pluginDeserializers === void 0 ? void 0 : pluginDeserializers(editor).leaf;
    if (!leafDeserializers) return;
    leafDeserializers.forEach(deserializer => {
      const leafPart = deserializer.deserialize(element);
      if (!leafPart) return;
      leaf = Object.assign(Object.assign({}, leaf), leafPart);
    });
  });
  return children.reduce((arr, child) => {
    if (!child) return arr;

    if (isElement(child)) {
      if (Object.keys(leaf).length) {
        slatePluginsCommon.mergeDeepToNodes({
          node: child,
          source: leaf,
          query: {
            filter: ([n]) => slate.Text.isText(n)
          }
        });
      }

      arr.push(child);
    } else {
      arr.push(slateHyperscript.jsx('text', leaf, child));
    }

    return arr;
  }, []);
};

/**
 * Deserialize HTML text node to text.
 */
const deserializeHTMLToText = node => {
  if (node.nodeType === Node.TEXT_NODE) {
    return node.nodeValue === '\n' ? null : node.textContent;
  }
};

/**
 * Deserialize HTML element or child node.
 */

const deserializeHTMLNode = (editor, plugins) => node => {
  var _a; // text node


  const textNode = deserializeHTMLToText(node);
  if (textNode) return textNode; // if not an element node

  if (node.nodeType !== Node.ELEMENT_NODE) return null;
  const htmlElement = node; // break line

  const breakLine = deserializeHTMLToBreak(node);
  if (breakLine) return breakLine;
  const {
    nodeName
  } = node;
  let parent = node; // blockquote

  if (nodeName === 'PRE' && ((_a = node.childNodes[0]) === null || _a === void 0 ? void 0 : _a.nodeName) === 'CODE') {
    [parent] = node.childNodes;
  }

  const children = Array.from(parent.childNodes).map(deserializeHTMLNode(editor, plugins)).flat(); // body

  const fragment = deserializeHTMLToFragment({
    element: htmlElement,
    children
  });
  if (fragment) return fragment; // element

  const element = deserializeHTMLToElement(editor, {
    plugins,
    element: htmlElement,
    children
  });
  if (element) return element; // mark

  return deserializeHTMLToMarks(editor, {
    plugins,
    element: htmlElement,
    children
  });
};

/**
 * Deserialize HTML element.
 */

const deserializeHTMLElement = (editor, {
  plugins,
  element
}) => {
  return deserializeHTMLNode(editor, plugins)(element);
};

/**
 * Deserialize HTML element to a valid document fragment.
 */

const deserializeHTMLToDocumentFragment = (editor, {
  plugins,
  element
}) => {
  if (typeof element === 'string') {
    element = htmlStringToDOMNode(element);
  }

  const fragment = deserializeHTMLElement(editor, {
    plugins,
    element
  });
  return slatePluginsCommon.normalizeDescendantsToDocumentFragment(fragment);
};

/**
 * Enables support for deserializing inserted content from HTML format to Slate format.
 */

const withDeserializeHTML = (_a = {}) => {
  var {
    plugins = []
  } = _a,
      options = __rest(_a, ["plugins"]);

  return editor => {
    const {
      insertData
    } = editor;
    const {
      preInsert = fragment => {
        const inlineTypes = getInlineTypes(editor, plugins);
        const firstNodeType = fragment[0].type; // replace the selected node type by the first block type

        if (slatePluginsCommon.isBlockAboveEmpty(editor) && firstNodeType && !inlineTypes.includes(firstNodeType)) {
          slate.Transforms.setNodes(editor, {
            type: fragment[0].type
          });
        }

        return fragment;
      },
      insert = fragment => {
        slate.Transforms.insertFragment(editor, fragment);
      }
    } = options;

    editor.insertData = data => {
      const html = data.getData('text/html');

      if (html) {
        const {
          body
        } = new DOMParser().parseFromString(html, 'text/html');
        let fragment = deserializeHTMLToDocumentFragment(editor, {
          plugins,
          element: body
        });
        fragment = preInsert(fragment);
        insert(fragment);
        return;
      }

      insertData(data);
    };

    return editor;
  };
};
/**
 * @see {@link withDeserializeHTML}
 */

const useDeserializeHTMLPlugin = getSlatePluginWithOverrides(withDeserializeHTML);

/**
 * Deserialize HTML to a valid Slate value.
 */

const deserializeHTMLToDocument = (editor, {
  plugins,
  element
}) => {
  const nodes = deserializeHTMLToDocumentFragment(editor, {
    plugins,
    element
  });
  return [{
    children: nodes
  }];
};

const trimWhitespace = rawHtml => rawHtml.replace(/(\r\n|\n|\r|\t)/gm, ''); // Remove redundant data attributes


const stripSlateDataAttributes = rawHtml => rawHtml.replace(/( data-slate)(-node|-type)="[^"]+"/gm, '').replace(/( data-testid)="[^"]+"/gm, '');
/**
 * Remove all class names that do not start with one of preserveClassNames (`slate-` by default)
 */


const stripClassNames = (html, {
  preserveClassNames = ['slate-']
}) => {
  const allClasses = html.split(/(class="[^"]*")/g);
  let filteredHtml = '';
  allClasses.forEach((item, index) => {
    if (index % 2 === 0) {
      return filteredHtml += item;
    }

    const preserveRegExp = new RegExp(preserveClassNames.map(cn => `${cn}[^"\\s]*`).join('|'), 'g');
    const slateClassNames = item.match(preserveRegExp);

    if (slateClassNames) {
      filteredHtml += `class="${slateClassNames.join(' ')}"`;
    }
  });
  return filteredHtml;
};

const getNode = (editor, {
  plugins,
  elementProps,
  slateProps,
  preserveClassNames
}) => {
  // If no type provided we wrap children with div tag
  if (!elementProps.element.type) {
    return `<div>${elementProps.children}</div>`;
  }

  let html; // Search for matching plugin based on element type

  plugins.some(plugin => {
    var _a, _b, _c, _d, _e, _f, _g;

    if (!((_a = plugin.serialize) === null || _a === void 0 ? void 0 : _a.element) && !plugin.renderElement) return false;

    if (!((_c = (_b = plugin.deserialize) === null || _b === void 0 ? void 0 : _b.call(plugin, editor).element) === null || _c === void 0 ? void 0 : _c.some(item => item.type === String(elementProps.element.type)))) {
      html = `<div>${elementProps.children}</div>`;
      return false;
    } // Render element using picked plugins renderElement function and ReactDOM


    html = server.renderToStaticMarkup(slatePluginsCommon.createElementWithSlate(Object.assign(Object.assign({}, slateProps), {
      children: (_f = (_e = (_d = plugin.serialize) === null || _d === void 0 ? void 0 : _d.element) === null || _e === void 0 ? void 0 : _e.call(_d, elementProps)) !== null && _f !== void 0 ? _f : (_g = plugin.renderElement) === null || _g === void 0 ? void 0 : _g.call(plugin, editor)(elementProps)
    })));
    html = stripClassNames(html, {
      preserveClassNames
    });
    return true;
  });
  return html;
};

const getLeaf = (editor, {
  plugins,
  leafProps,
  slateProps,
  preserveClassNames
}) => {
  const {
    children
  } = leafProps;
  return plugins.reduce((result, plugin) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;

    if (!((_a = plugin.serialize) === null || _a === void 0 ? void 0 : _a.leaf) && !plugin.renderLeaf) return result;
    if (((_d = (_c = (_b = plugin.serialize) === null || _b === void 0 ? void 0 : _b.leaf) === null || _c === void 0 ? void 0 : _c.call(_b, leafProps)) !== null && _d !== void 0 ? _d : (_e = plugin.renderLeaf) === null || _e === void 0 ? void 0 : _e.call(plugin, editor)(leafProps)) === children) return result;
    const newLeafProps = Object.assign(Object.assign({}, leafProps), {
      children: encodeURIComponent(result)
    });
    let html = decodeURIComponent(server.renderToStaticMarkup(slatePluginsCommon.createElementWithSlate(Object.assign(Object.assign({}, slateProps), {
      children: (_h = (_g = (_f = plugin.serialize) === null || _f === void 0 ? void 0 : _f.leaf) === null || _g === void 0 ? void 0 : _g.call(_f, leafProps)) !== null && _h !== void 0 ? _h : (_j = plugin.renderLeaf) === null || _j === void 0 ? void 0 : _j.call(plugin, editor)(newLeafProps)
    }))));
    html = stripClassNames(html, {
      preserveClassNames
    });
    return html;
  }, children);
};

const isEncoded = (str = '') => {
  try {
    return str !== decodeURIComponent(str);
  } catch (error) {
    return false;
  }
};
/**
 * Convert Slate Nodes into HTML string
 */


const serializeHTMLFromNodes = (editor, {
  plugins,
  nodes,
  slateProps,
  stripDataAttributes = true,
  preserveClassNames
}) => {
  let result = nodes.map(node => {
    if (slate.Text.isText(node)) {
      return getLeaf(editor, {
        plugins,
        leafProps: {
          leaf: node,
          text: node,
          children: isEncoded(node.text) ? node.text : encodeURIComponent(node.text),
          attributes: {
            'data-slate-leaf': true
          }
        },
        slateProps,
        preserveClassNames
      });
    }

    return getNode(editor, {
      plugins,
      elementProps: {
        element: node,
        children: encodeURIComponent(serializeHTMLFromNodes(editor, {
          plugins,
          nodes: node.children,
          preserveClassNames
        })),
        attributes: {
          'data-slate-node': 'element',
          ref: null
        }
      },
      slateProps,
      preserveClassNames
    });
  }).join('');
  result = trimWhitespace(decodeURIComponent(result));

  if (stripDataAttributes) {
    result = stripSlateDataAttributes(result);
  }

  return result;
};

exports.deserializeHTMLElement = deserializeHTMLElement;
exports.deserializeHTMLNode = deserializeHTMLNode;
exports.deserializeHTMLToBreak = deserializeHTMLToBreak;
exports.deserializeHTMLToDocument = deserializeHTMLToDocument;
exports.deserializeHTMLToDocumentFragment = deserializeHTMLToDocumentFragment;
exports.deserializeHTMLToElement = deserializeHTMLToElement;
exports.deserializeHTMLToFragment = deserializeHTMLToFragment;
exports.deserializeHTMLToMarks = deserializeHTMLToMarks;
exports.deserializeHTMLToText = deserializeHTMLToText;
exports.htmlStringToDOMNode = htmlStringToDOMNode;
exports.serializeHTMLFromNodes = serializeHTMLFromNodes;
exports.useDeserializeHTMLPlugin = useDeserializeHTMLPlugin;
exports.withDeserializeHTML = withDeserializeHTML;
//# sourceMappingURL=index.js.map
